@@ -10,7 +10,7 @@ require Exporter;
 require List::Util; # List::Util loads the XS
 
 @ISA       = qw(Exporter);
-@EXPORT_OK = qw(blessed dualvar reftype weaken isweak tainted readonly);
+@EXPORT_OK = qw(blessed dualvar reftype weaken isweak tainted readonly openhandle refaddr isvstring);
 $VERSION   = $VERSION = $List::Util::VERSION;
 
 sub export_fail {
@@ -18,6 +18,10 @@ sub export_fail {
     require Carp;
     Carp::croak("Weak references are not implemented in the version of perl");
   }
+  if (grep { /^(isvstring)$/ } @_ ) {
+    require Carp;
+    Carp::croak("Vstrings are not implemented in the version of perl");
+  }
   if (grep { /^dualvar$/ } @_ ) {
     require Carp;
     Carp::croak("dualvar is only avaliable with the XS version");
@@ -26,9 +30,27 @@ sub export_fail {
   @_;
 }
 
+sub openhandle ($) {
+  my $fh = shift;
+  my $rt = reftype($fh) || '';
+
+  return defined(fileno($fh)) ? $fh : undef
+    if $rt eq 'IO';
+
+  if (reftype(\$fh) eq 'GLOB') { # handle  openhandle(*DATA)
+    $fh = \(my $tmp=$fh);
+  }
+  elsif ($rt ne 'GLOB') {
+    return undef;
+  }
+
+  (tied(*$fh) or defined(fileno($fh)))
+    ? $fh : undef;
+}
+
 eval <<'ESQ' unless defined &dualvar;
 
-push @EXPORT_FAIL, qw(weaken isweak dualvar);
+push @EXPORT_FAIL, qw(weaken isweak dualvar isvstring);
 
 # The code beyond here is only used if the XS is not installed
 
@@ -42,6 +64,14 @@ sub blessed ($) {
     : undef
 }
 
+sub refaddr($) {
+  my $pkg = ref($_[0]) or return undef;
+  bless $_[0], 'Scalar::Util::Fake';
+  my $i = int($_[0]);
+  bless $_[0], $pkg;
+  $i;
+}
+
 sub reftype ($) {
   local($@, $SIG{__DIE__}, $SIG{__WARN__});
   my $r = shift;
@@ -99,7 +129,7 @@ Scalar::Util - A selection of general-utility scalar subroutines
 
 =head1 SYNOPSIS
 
-    use Scalar::Util qw(blessed dualvar isweak readonly reftype tainted weaken);
+    use Scalar::Util qw(blessed dualvar isweak readonly refaddr reftype tainted weaken);
 
 =head1 DESCRIPTION
 
@@ -136,6 +166,14 @@ value STRING in a string context.
     $num = $foo + 2;                    # 12
     $str = $foo . " world";             # Hello world
 
+=item isvstring EXPR
+
+If EXPR is a scalar which was coded as a vstring the result is true.
+
+    $vs   = v49.46.48;
+    $fmt  = isvstring($vs) ? "%vd" : "%s"; #true
+    printf($fmt,$vs);
+
 =item isweak EXPR
 
 If EXPR is a scalar which is a weak reference the result is true.
@@ -145,6 +183,16 @@ If EXPR is a scalar which is a weak reference the result is true.
     weaken($ref);
     $weak = isweak($ref);               # true
 
+=item openhandle FH
+
+Returns FH if FH may be used as a filehandle and is open, or FH is a tied
+handle. Otherwise C<undef> is returned.
+
+    $fh = openhandle(*STDIN);		# \*STDIN
+    $fh = openhandle(\*STDIN);		# \*STDIN
+    $fh = openhandle(*NOTOPEN);		# undef
+    $fh = openhandle("scalar");		# undef
+    
 =item readonly SCALAR
 
 Returns true if SCALAR is readonly.
@@ -154,6 +202,18 @@ Returns true if SCALAR is readonly.
     $readonly = foo($bar);              # false
     $readonly = foo(0);                 # true
 
+=item refaddr EXPR
+
+If EXPR evaluates to a reference the internal memory address of
+the referenced value is returned. Otherwise C<undef> is returned.
+
+    $addr = refaddr "string";           # undef
+    $addr = refaddr \$var;              # eg 12345678
+    $addr = refaddr [];                 # eg 23456784
+
+    $obj  = bless {}, "Foo";
+    $addr = refaddr $obj;               # eg 88123488
+
 =item reftype EXPR
 
 If EXPR evaluates to a reference the type of the variable referenced
@@ -191,6 +251,11 @@ prevent the object being DESTROY-ed at its usual time.
 
 =back
 
+=head1 KNOWN BUGS
+
+There is a bug in perl5.6.0 with UV's that are >= 1<<31. This will
+show up as tests 8 and 9 of dualvar.t failing
+
 =head1 COPYRIGHT
 
 Copyright (c) 1997-2001 Graham Barr <gbarr@pobox.com>. All rights reserved.
